@@ -71,6 +71,7 @@ class Forecast(Workflow, ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(Forecast, cls).__setup__()
+        cls.create_date.select = True
         cls._order.insert(0, ('from_date', 'DESC'))
         cls._order.insert(1, ('warehouse', 'ASC'))
         cls._transitions |= set((
@@ -106,9 +107,7 @@ class Forecast(Workflow, ModelSQL, ModelView):
 
         super(Forecast, cls).__register__(module_name)
 
-        # Add index on create_date
         table = cls.__table_handler__(module_name)
-        table.index_action('create_date', action='add')
 
         # Migration from 5.0: remove check_from_to_date
         table.drop_constraint('check_from_to_date')
@@ -178,7 +177,7 @@ class Forecast(Workflow, ModelSQL, ModelView):
             return
         transaction = Transaction()
         connection = transaction.connection
-        transaction.database.lock(connection, self._table)
+        self.__class__.lock()
         forcast = self.__table__()
         cursor = connection.cursor()
         cursor.execute(*forcast.select(forcast.id,
@@ -279,16 +278,14 @@ class ForecastLine(ModelSQL, ModelView):
             ],
         states=_states,
         depends=['product', 'product_uom_category'] + _depends)
-    unit_digits = fields.Function(fields.Integer('Unit Digits'),
-            'get_unit_digits')
     quantity = fields.Float(
-        "Quantity", digits=(16, Eval('unit_digits', 2)), required=True,
+        "Quantity", digits='uom', required=True,
         domain=[('quantity', '>=', 0)],
-        states=_states, depends=['unit_digits'] + _depends)
+        states=_states, depends=_depends)
     minimal_quantity = fields.Float(
-        "Minimal Qty", digits=(16, Eval('unit_digits', 2)), required=True,
+        "Minimal Qty", digits='uom', required=True,
         domain=[('minimal_quantity', '<=', Eval('quantity'))],
-        states=_states, depends=['unit_digits', 'quantity'] + _depends)
+        states=_states, depends=['quantity'] + _depends)
     moves = fields.Many2Many('stock.forecast.line-stock.move',
         'line', 'move', 'Moves', readonly=True)
     forecast = fields.Many2One(
@@ -301,15 +298,15 @@ class ForecastLine(ModelSQL, ModelView):
     forecast_state = fields.Function(
         fields.Selection('get_forecast_states', 'Forecast State'),
         'on_change_with_forecast_state')
-    quantity_executed = fields.Function(fields.Float('Quantity Executed',
-            digits=(16, Eval('unit_digits', 2)), depends=['unit_digits']),
-        'get_quantity_executed')
+    quantity_executed = fields.Function(fields.Float(
+            "Quantity Executed", digits='uom'), 'get_quantity_executed')
 
     del _states, _depends
 
     @classmethod
     def __setup__(cls):
         super(ForecastLine, cls).__setup__()
+        cls.__access__.add('forecast')
         t = cls.__table__()
         cls._sql_constraints += [
             ('forecast_product_uniq', Unique(t, t.forecast, t.product),
@@ -326,35 +323,20 @@ class ForecastLine(ModelSQL, ModelView):
         table_h.drop_constraint('check_line_qty_pos')
         table_h.drop_constraint('check_line_minimal_qty')
 
-    @staticmethod
-    def default_unit_digits():
-        return 2
-
     @staticmethod
     def default_minimal_quantity():
         return 1.0
 
     @fields.depends('product')
     def on_change_product(self):
-        self.unit_digits = 2
         if self.product:
             self.uom = self.product.default_uom
-            self.unit_digits = self.product.default_uom.digits
 
     @fields.depends('product')
     def on_change_with_product_uom_category(self, name=None):
         if self.product:
             return self.product.default_uom_category.id
 
-    @fields.depends('uom')
-    def on_change_uom(self):
-        self.unit_digits = 2
-        if self.uom:
-            self.unit_digits = self.uom.digits
-
-    def get_unit_digits(self, name):
-        return self.product.default_uom.digits
-
     @classmethod
     def get_forecast_states(cls):
         pool = Pool()
@@ -418,7 +400,7 @@ class ForecastLine(ModelSQL, ModelView):
                             <= forecast.to_date)
                         & (line_move.id == Null),
                         group_by=move.product))
-                for product_id, quantity in cursor.fetchall():
+                for product_id, quantity in cursor:
                     line = product2line[product_id]
                     result[line.id] = Uom.compute_qty(line.product.default_uom,
                         quantity, line.uom)
@@ -457,7 +439,7 @@ class ForecastLine(ModelSQL, ModelView):
         distribution = self.distribute(delta, nb_packet)
         unit_price = None
         if self.forecast.destination.type == 'customer':
-            unit_price = self.product.list_price
+            unit_price = self.product.list_price or 0
             unit_price = Uom.compute_price(self.product.default_uom,
                 unit_price, self.uom)
 
